/*
 * cpu.cc
 *
 * Description of this file:
 *    CPU-Processing functions definition of the xrddec library
 *
 * --------------------------------------------------------------------------
 *
 *    xrddec - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */

#include "common.h"
#include "cpu.h"

#if MACOS || SYS_FREEBSD
#include <sys/types.h>
#include <sys/sysctl.h>
#endif
#if SYS_OPENBSD
#include <sys/param.h>
#include <sys/sysctl.h>
#include <machine/cpu.h>
#endif

#if !ARCH_X86_64
/*  */
int  avsd_cpu_cpuid_test(void);
#endif

#if SYS_LINUX && !(defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_7__))
/* ---------------------------------------------------------------------------
 */
int sched_getaffinity(pid_t pid, size_t cpusetsize, cpu_set_t *mask);
#endif

/* ---------------------------------------------------------------------------
 */
int xrddec_cpu_num_processors(void)
{
#if !HAVE_THREAD
    return 1;
#elif defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_7__)
    return 2;
#elif SYS_WINDOWS
    return xrddec_thread_num_processors_np();
#elif SYS_LINUX
    unsigned int bit;
    int np = 0;
    cpu_set_t p_aff;

    memset(&p_aff, 0, sizeof(p_aff));
    sched_getaffinity(0, sizeof(p_aff), &p_aff);
    for (bit = 0; bit < sizeof(p_aff); bit++) {
        np += (((uint8_t *)& p_aff)[bit / 8] >> (bit % 8)) & 1;
    }
    return np;

#elif SYS_BEOS
    system_info info;

    get_system_info(&info);
    return info.cpu_count;

#elif SYS_MACOSX || SYS_FREEBSD || SYS_OPENBSD
    int numberOfCPUs;
    size_t length = sizeof (numberOfCPUs);
#if SYS_OPENBSD
    int mib[2] = { CTL_HW, HW_NCPU };
    if(sysctl(mib, 2, &numberOfCPUs, &length, NULL, 0))
#else
    if(sysctlbyname("hw.ncpu", &numberOfCPUs, &length, NULL, 0))
#endif
    {
        numberOfCPUs = 1;
    }
    return numberOfCPUs;

#else
    return 1;
#endif
}
