/*
 * frame.h
 *
 * Description of this file:
 *    Frame handling functions definition of the xRD library
 *
 * --------------------------------------------------------------------------
 *
 *    xRD - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */

#ifndef __XRD_FRAME_H__
#define __XRD_FRAME_H__


/**
 * ===========================================================================
 * function declares
 * ===========================================================================
 */

xrd_frame_t *xrd_frame_new(xrd_t *h, uint8_t **mem_base, int alloc_type);
void xrd_frame_delete(xrd_handler_t *h_mgr, xrd_frame_t *frame);

int xrd_frame_buffer_size(xrd_param_t *param, int alloc_type);
void xrd_frame_destroy_objects(xrd_handler_t *h_mgr, xrd_frame_t *frame);

void xrd_frame_copy_planes(xrd_t *h, xrd_frame_t *dst, xrd_frame_t *src);

void plane_expand_border(pel_t *p_pix, int i_stride, int i_width, int i_height,
                         int i_padh, int i_padv, int b_pad_top, int b_pad_bottom); 
void xrd_frame_expand_border_frame(xrd_t *h, xrd_frame_t *frame);
void xrd_frame_expand_border_lcurow(xrd_t *h, xrd_frame_t *frame, int i_lcu_y);

void xrd_frame_expand_border_filtered(xrd_frame_t *frame, int i_lcu_y, int lcu_h, int b_end);
void xrd_frame_expand_border_mod8(xrd_t *h, xrd_frame_t *frame);

int xrd_lowres_buffer_size(xrd_param_t *param);
frm_lowres_t *xrd_lowres_new(xrd_t *h, uint8_t **mem_base);
void xrd_frame_expand_border_lowres(frm_lowres_t *lowres);

void xrd_frame_init_lowres(frm_lowres_t *lowres, xrd_frame_t *frm);
void xrd_frame_lowres_copy(frm_lowres_t *dst_frm, frm_lowres_t *src_frm);

#endif  /* __XRD_FRAME_H__ */
