/*
 * mc.h
 *
 * Description of this file:
 *    MC functions definition of the xRD library
 *
 * --------------------------------------------------------------------------
 *
 *    xRD - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */

#ifndef __XRD_MC_H__
#define __XRD_MC_H__

/**
 * ===========================================================================
 * inline function declares
 * ===========================================================================
 */

/* ---------------------------------------------------------------------------
 * img_size: ؾȵͼ Ȼ߶ ؾȣ
 * blk_size: ǰԤ Ȼ߶     ؾȣ
 * blk_pos:  ǰͼе x/y    ؾȣ
 * mv     :  MV  x/y              1/4ؾȣ
 */
static INLINE
int cu_get_mc_pos(int img_size, int blk_size, int blk_pos, int mv)
{
    // WARNING: 10bit²ֵʱλڼȨϵ֮ͣ
    // £±߽4xyͷصֵx(xΪ)صֵһ£
    //           ұ߽4xyͷصֵy(yΪ)صֵһ¡
    // Ϊ֤ƥ䣬ҪMV1/8ȽClipͬ8bit
    int imv = mv >> 2;  // MVؾ
    int fmv = mv & 7;   // MVķؾȲ֣ 1/8 

    if (blk_pos + imv < -blk_size - 8) {
        return ((-blk_size - 8) << 2) + (fmv);
    } else if (blk_pos + imv > img_size + 4) {
        return ((img_size + 4) << 2) + (fmv);
    } else {
        return (blk_pos << 2) + mv;
    }
}

/* ---------------------------------------------------------------------------
 */
static ALWAYS_INLINE
void get_mv_for_mc(xrd_t *h, mv_t *mv, int pic_pix_x, int pic_pix_y, int blk_w, int blk_h)
{
    // WARNING: ͼֱΪ 4K ʱ㹻8K ʱ
    mv->x = (int16_t)cu_get_mc_pos(h->i_width,  blk_w, pic_pix_x, mv->x);
    mv->y = (int16_t)cu_get_mc_pos(h->i_height, blk_h, pic_pix_y, mv->y);
}

/**
 * ===========================================================================
 * function declares
 * ===========================================================================
 */
void interpolate_lcu_row(xrd_t *h, xrd_frame_t* frm, int i_lcu_y);

void interpolate_sample_rows(xrd_t *h, xrd_frame_t* frm, int start_y, int height, int b_start, int b_end);

void mc_luma  (pel_t *p_pred, int i_pred, 
               int pic_pix_x, int pic_pix_y, int width, int height, 
               const xrd_frame_t *p_ref_frm);
void mc_chroma(pel_t *p_pred_u, pel_t *p_pred_v, int i_pred, 
               int pix_quad_x, int pix_quad_y, int width, int height,
               const xrd_frame_t *p_ref_frm);


#endif // __XRD_MC_H__
