/*
 * primitives.c
 *
 * Description of this file:
 *    function handles initialize functions definition of the xRD library
 *
 * --------------------------------------------------------------------------
 *
 *    xRD - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */

#include "common.h"
#include "primitives.h"
#include "cpu.h"
#include "intra.h"
#include "mc.h"
#include "transform.h"
#include "filter.h"
#include "sao.h"

/* ---------------------------------------------------------------------------
 * global function handle
 */
intrinsic_func_t g_funcs;

/* ---------------------------------------------------------------------------
 */
void init_all_primitives(xrd_param_t* param, intrinsic_func_t *p_funcs)
{
    if (param != NULL) {
        /* bit depth for encoding */
        g_bit_depth   = param->sample_bit_depth;
        max_pel_value = (1 << g_bit_depth) - 1;
        g_dc_value    = (1 << g_bit_depth) >> 1;
    }

    /* init memory operation function handlers */
    xrd_mem_oper_init();

    /* init function handles */
    xrd_intra_pred_init(p_funcs);
    xrd_mc_init        (p_funcs);
    xrd_pixel_init     (&p_funcs->pixf);
    xrd_deblock_init   (p_funcs);
    xrd_dct_init       (&p_funcs->dctf);
    xrd_quant_init     (&p_funcs->dctf);
    xrd_cg_scan_init   (p_funcs);
    xrd_mad_init       (p_funcs->pixf.madf);

    xrd_sao_init       (p_funcs);
    xrd_alf_init       (p_funcs);

    xrd_rdo_init       (p_funcs);
}
