/*
 * win32thread.h
 *
 * Description of this file:
 *    windows threading of the xRD library
 *
 * --------------------------------------------------------------------------
 *
 *    xRD - video encoder of AVS2/IEEE1857.4 video coding standard
 *    Copyright (C) 2018~ VCL, NELVT, Peking University
 *
 *    Authors: Falei LUO <falei.luo@gmail.com>
 *             etc.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *
 *    This program is also available under a commercial proprietary license.
 *    For more information, contact us at sswang @ pku.edu.cn.
 */

#ifndef __XRD_WIN32THREAD_H__
#define __XRD_WIN32THREAD_H__

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
/* the following macro is used within XRDENC */
#undef ERROR

typedef struct {
    void *handle;
    void *(*func)(void *arg);
    void *arg;
    void *ret;
} xrd_pthread_t;
#define xrd_pthread_attr_t int

/* the conditional variable api for windows 6.0+ uses critical sections and not mutexes */
typedef CRITICAL_SECTION xrd_pthread_mutex_t;
#define XRD_PTHREAD_MUTEX_INITIALIZER {0}
#define xrd_pthread_mutexattr_t int
#define pthread_exit(a)
/* This is the CONDITIONAL_VARIABLE typedef for using Window's native conditional variables on kernels 6.0+.
 * MinGW does not currently have this typedef. */
typedef struct {
    void *ptr;
} xrd_pthread_cond_t;
#define xrd_pthread_condattr_t int

int xrd_pthread_create(xrd_pthread_t *thread, const xrd_pthread_attr_t *attr,
                        void *(*start_routine)(void *), void *arg);
int xrd_pthread_join(xrd_pthread_t thread, void **value_ptr);

int xrd_pthread_mutex_init(xrd_pthread_mutex_t *mutex, const xrd_pthread_mutexattr_t *attr);
int xrd_pthread_mutex_destroy(xrd_pthread_mutex_t *mutex);
int xrd_pthread_mutex_lock(xrd_pthread_mutex_t *mutex);
int xrd_pthread_mutex_unlock(xrd_pthread_mutex_t *mutex);

int xrd_pthread_cond_init(xrd_pthread_cond_t *cond, const xrd_pthread_condattr_t *attr);
int xrd_pthread_cond_destroy(xrd_pthread_cond_t *cond);
int xrd_pthread_cond_broadcast(xrd_pthread_cond_t *cond);
int xrd_pthread_cond_wait(xrd_pthread_cond_t *cond, xrd_pthread_mutex_t *mutex);
int xrd_pthread_cond_signal(xrd_pthread_cond_t *cond);

#define xrd_pthread_attr_init(a) 0
#define xrd_pthread_attr_destroy(a) 0

int  xrd_win32_threading_init(void);
void xrd_win32_threading_destroy(void);

int xrd_pthread_num_processors_np(void);

#endif  // __XRD_WIN32THREAD_H__
